@@ -33,6 +33,8 @@ class S3FSOpener(Opener):
             aws_access_key_id=parse_result.username or None,
             aws_secret_access_key=parse_result.password or None,
             endpoint_url=parse_result.params.get('endpoint_url', None),
+            acl=parse_result.params.get('acl', None),
+            cache_control=parse_result.params.get('cache_control', None),
             strict=strict
         )
         return s3fs
